@@ -3,7 +3,6 @@ import pandas as pd
 import math
 import matplotlib.pyplot as plt
 import yfinance as yf
-from pandas_datareader import data as web
 import datetime as dt
 from empyrical import*
 import quantstats as qs
@@ -13,6 +12,7 @@ import matplotlib.pyplot as plt
 import copy
 import yfinance as yf
 import warnings
+from EfficientFrontier import efficient_frontier
 
 warnings.filterwarnings("ignore")
 
@@ -22,11 +22,10 @@ warnings.filterwarnings("ignore")
 today = dt.date.today()
 
 # ------------------------------------------------------------------------------------------
-
 class Engine:
 
 
-  def __init__(self,start_date, portfolio, weights=None, rebalance=None, benchmark=['SPY'], end_date=today, optimizer=None, max_vol=0.15, diversification=1,confidences=None, view=None, min_weights=None, max_weights=None):
+  def __init__(self,start_date, portfolio, weights=None, rebalance=None, benchmark=['SPY'], end_date=today, optimizer=None, max_vol=0.15, diversification=1,confidences=None, view=None, min_weights=None, max_weights=None, risk_manager=None):
     self.start_date = start_date
     self.end_date = end_date
     self.portfolio = portfolio
@@ -34,6 +33,7 @@ class Engine:
     self.benchmark = benchmark
     self.optimizer = optimizer
     self.rebalance = rebalance
+    self.risk_manager = risk_manager
     self.max_vol = max_vol
     self.diversification = diversification
     self.view = view
@@ -59,144 +59,23 @@ class Engine:
     if self.optimizer== "BL":
       self.weights = bl(self, perf="False")
       
-    if self.rebalance!=None & self.optimizer !="BL":
-      self.rebalance = make_rebalance(self.start_date, self.end_date, self.optimizer, self.portfolio, self.rebalance, self.max_vol, self.weights)
-
-    if self.rebalance!=None & self.optimizer=="BL":
-      self.rebalance = make_rebalance(self.start_date, self.end_date, False, self.portfolio, self.rebalance, self.max_vol, self.weights)
-
-    if self.rebalance!=None & self.optimizer==None:
-      self.rebalance = make_rebalance(self.start_date, self.end_date, False, self.portfolio, self.rebalance, self.max_vol, self.weights)
-    
+    if self.rebalance != None:
+      self.rebalance = make_rebalance(self.start_date, self.end_date, self.optimizer, self.portfolio, self.rebalance, self.weights, self.max_vol, self.diversification, self.min_weights, self.max_weights)
 #-------------------------------------------------------------------------------------------
 def get_returns(stocks,wts, start_date, end_date=today):
     
   if len(stocks) > 1:
-    assets = web.DataReader(stocks, data_source='yahoo', start = start_date, end= end_date)['Adj Close']
+    assets = yf.download(stocks, start=start_date, end=end_date, progress=False)['Adj Close']
+    assets = assets.filter(stocks)
     ret_data = assets.pct_change()[1:]
     returns = (ret_data * wts).sum(axis = 1)
     return returns
   else:
-    df = web.DataReader(stocks, data_source='yahoo', start = start_date, end= end_date)['Adj Close']
+    df = yf.download(stocks, start=start_date, end=end_date, progress=False)['Adj Close']
     df = pd.DataFrame(df)
     returns = df.pct_change()
     return returns
 # ------------------------------------------------------------------------------------------
-def get_pricing(stocks, start_date, end_date=today, pricing="Adj Close", wts=1):
-    
-  if len(stocks) > 1:
-    assets = web.DataReader(stocks, data_source='yahoo', start = start_date, end= end_date)[pricing]
-    return assets
-  else:
-    df = web.DataReader(stocks, data_source='yahoo', start = start_date, end= end_date)[pricing]
-    return df
-# ------------------------------------------------------------------------------------------
-
-def get_data(stocks, period="max", trading_year_days=252):
-
-  p = {"period": period}
-  for stock in stocks:
-    years = {
-      '1mo' : math.ceil(trading_year_days/12),
-      '3mo' : math.ceil(trading_year_days/4),
-      '6mo' : math.ceil(trading_year_days/2),
-      '1y': trading_year_days,
-      '2y' : 2*trading_year_days,
-      '5y' : 5*trading_year_days,
-      '10y' : 10*trading_year_days,
-      '20y' : 20*trading_year_days,
-      'max' : len(yf.Ticker(stock).history(**p)['Close'].pct_change())
-    }
-
-  df = web.DataReader(stocks, data_source='yahoo', start = "1980-01-01", end= today)
-  df = pd.DataFrame(df)
-  df = df.tail(years[period])
-  df = pd.DataFrame(df)
-  df = df.drop(['Adj Close'], axis=1)
-  df = df[["Open", "High", "Low", "Close", "Volume"]]
-  return df
-
-# ------------------------------------------------------------------------------------------
-
-#reformat
-def creturns(stocks,wts=1, period="max", benchmark= None, plot=True, pricing="Adj Close", trading_year_days=252, end_date = today):
-  p = {"period": period}
-  for stock in stocks:
-    years = {
-      '1mo' : math.ceil(trading_year_days/12),
-      '3mo' : math.ceil(trading_year_days/4),
-      '6mo' : math.ceil(trading_year_days/2),
-      '1y': trading_year_days,
-      '2y' : 2*trading_year_days,
-      '5y' : 5*trading_year_days,
-      '10y' : 10*trading_year_days,
-      '20y' : 20*trading_year_days,
-      'max' : len(yf.Ticker(stock).history(**p)['Close'].pct_change())
-    }
-
-  if len(stocks) > 1:
-    df = web.DataReader(stocks, data_source='yahoo', start = "1980-01-01", end= end_date)[pricing]
-    if benchmark != None:
-      df2 = web.DataReader(benchmark, data_source='yahoo', start = "1980-01-01", end= end_date)[pricing]
-      df = pd.DataFrame(df)
-      df = df.tail(years[period])
-      df2 = df2.tail(years[period])
-      return_df2 = df2.pct_change()[1:]
-      ret_data = df.pct_change()[1:]
-      ret_data = (ret_data + 1).cumprod()
-      port_ret = (ret_data * wts).sum(axis = 1)
-      return_df2 = (return_df2 + 1).cumprod()
-      ret_data['Portfolio'] = port_ret
-      ret_data['Benchmark'] = return_df2
-      ret_data = pd.DataFrame(ret_data)
-    else:
-      df = pd.DataFrame(df)
-      df = df.tail(years[period])
-      ret_data = df.pct_change()[1:]
-      ret_data = (ret_data + 1).cumprod()
-      port_ret = (ret_data * wts).sum(axis = 1)
-      ret_data['Portfolio'] = port_ret
-      ret_data = pd.DataFrame(ret_data)
-
-    if plot==True:
-      ret_data.plot(figsize=(20,10))
-      plt.xlabel('Date')
-      plt.ylabel('Returns')
-      plt.title(period + 'Portfolio Cumulative Returns')
-    else:
-      return ret_data
-  else:
-    df = web.DataReader(stocks, data_source='yahoo', start = "1980-01-01", end= today)[pricing]
-    if benchmark != None:
-      df2 = web.DataReader(benchmark, data_source='yahoo', start = "1980-01-01", end= today)[pricing]
-      return_df2 = df2.pct_change()[1:]
-      df = pd.DataFrame(df)
-      df = df.tail(years[period])
-      df2 = df2.tail(years[period])
-      return_df2 = df2.pct_change()[1:]
-      returns = df.pct_change()
-      returns = (returns + 1).cumprod()
-      return_df2 = (return_df2 + 1).cumprod()
-      returns["benchmark"] = return_df2
-      returns = pd.DataFrame(returns)
-    else:
-      df = pd.DataFrame(df)
-      df = df.tail(years[period])
-      returns = df.pct_change()
-      returns = (returns + 1).cumprod()
-      returns = pd.DataFrame(returns)
-
-    if plot==True:
-        returns.plot(figsize=(20,10))
-        plt.axvline(x=1)
-        plt.xlabel('Date')
-        plt.ylabel('Returns')
-        plt.title(stocks[0] +' Cumulative Returns (Period : '+ period+')')
-
-    else:
-        return returns
-
-# ------------------------------------------------------------------------------------------
 def information_ratio(returns, benchmark_returns, days=252):
  return_difference = returns - benchmark_returns
  volatility = return_difference.std() * np.sqrt(days)
@@ -229,19 +108,32 @@ def empyrial(my_portfolio, rf=0.0, sigma_value=1, confidence_value=0.95, rebalan
       
       #we want to get the dataframe with the dates and weights
       rebalance_schedule = my_portfolio.rebalance
+
+      columns = []
+      for date in rebalance_schedule.columns:
+            date = date[0:10]
+            columns.append(date)
+      rebalance_schedule.columns = columns
       
       #then want to make a list of the dates and start with our first date
       dates = [my_portfolio.start_date]
       
       #then our rebalancing dates into that list 
       dates = dates + rebalance_schedule.columns.to_list()
-      
+
+      datess = []
+      for date in dates:
+          date = date[0:10]
+          datess.append(date)
+      dates = datess
       #this will hold returns
       returns = pd.Series()
       
+  
       #then we want to be able to call the dates like tuples
       for i in range(len(dates) - 1):
           
+
           #get our weights
           weights = rebalance_schedule[str(dates[i+1])]
           
@@ -250,7 +142,73 @@ def empyrial(my_portfolio, rf=0.0, sigma_value=1, confidence_value=0.95, rebalan
           
           #then append those returns
           returns = returns.append(add_returns)
+
+  creturns = (returns + 1).cumprod()
+
+  #risk manager
+  try:
+    if list(my_portfolio.risk_manager.keys())[0] == "Stop Loss":
+
+      values = []
+      for r in creturns:
+        if r <= 1+my_portfolio.risk_manager['Stop Loss']:
+          values.append(r)
+        else:
+          pass
+
+      try:
+        date = creturns[creturns == values[0]].index[0]
+        date = str(date.to_pydatetime())
+        my_portfolio.end_date = date[0:10]
+        returns = returns[:my_portfolio.end_date]
+
+      except Exception as e:
+        pass
+
+    if list(my_portfolio.risk_manager.keys())[0] == "Take Profit":
+
+      values = []
+      for r in creturns:
+        if r >= 1+my_portfolio.risk_manager['Take Profit']:
+          values.append(r)
+        else:
+          pass
+      
+      try:
+        date = creturns[creturns == values[0]].index[0]
+        date = str(date.to_pydatetime())
+        my_portfolio.end_date = date[0:10]
+        returns = returns[:my_portfolio.end_date]
+
+      except Exception as e:
+        pass
+
+    if list(my_portfolio.risk_manager.keys())[0] == "Max Drawdown":
+
+      drawdown = qs.stats.to_drawdown_series(returns)
+
+      values = []
+      for r in drawdown:
+        if r <= my_portfolio.risk_manager['Max Drawdown']:
+          values.append(r)
+        else:
+          pass
+
+      try:
+        date = drawdown[drawdown == values[0]].index[0]
+        date = str(date.to_pydatetime())
+        my_portfolio.end_date = date[0:10]
+        returns = returns[:my_portfolio.end_date]
+
+      except Exception as e:
+        pass
+
+  except Exception as e:
+      pass
+      
   
+  print("Start date: " + str(my_portfolio.start_date))
+  print("End date: " + str(my_portfolio.end_date))
 
   benchmark = get_returns(my_portfolio.benchmark, wts=[1], start_date=my_portfolio.start_date,end_date=my_portfolio.end_date)
 
@@ -290,7 +248,7 @@ def empyrial(my_portfolio, rf=0.0, sigma_value=1, confidence_value=0.95, rebalan
 
 
   MD = max_drawdown(returns, out=None)
-  MD = str(round(MD,2))+' %'
+  MD = str(round(MD*100,2))+' %'
 
   
   '''OR = omega_ratio(returns, risk_free=0.0, required_return=0.0)
@@ -383,6 +341,7 @@ def empyrial(my_portfolio, rf=0.0, sigma_value=1, confidence_value=0.95, rebalan
   plt.title('Returns')
 
   empyrial.returns = returns
+  empyrial.creturns = creturns
   empyrial.benchmark = benchmark
   empyrial.CAGR = CAGR
   empyrial.CUM = CUM
@@ -425,7 +384,6 @@ def flatten(seq):
             l.append(elt)
     return l
 #-------------------------------------------------------------------------------
-
 def graph_opt(my_portfolio, my_weights, pie_size, font_size):
   fig1, ax1 = plt.subplots()
   fig1.set_size_inches(pie_size, pie_size)
@@ -439,46 +397,15 @@ def graph_opt(my_portfolio, my_weights, pie_size, font_size):
 def equal_weighting(my_portfolio):
   return [1.0/len(my_portfolio.portfolio)]*len(my_portfolio.portfolio)
 #------------------------------------------------------------------------
-def efficient_frontier(my_portfolio, perf=True):
 
-  #changed to take in desired timeline, the problem is that it would use all historical data
-  ohlc = yf.download(my_portfolio.portfolio, start = my_portfolio.start_date, end = my_portfolio.end_date, progress=False)
-  prices = ohlc["Adj Close"].dropna(how="all")
-  df = prices.filter(my_portfolio.portfolio)
-  
-  #sometimes we will pick a date range where company isn't public we can't set price to 0 so it has to go to 1
-  df = df.fillna(1)
-
-  mu = expected_returns.mean_historical_return(df)
-  S = risk_models.sample_cov(df)
-
-  #optimize for max sharpe ratio
-  ef = EfficientFrontier(mu, S)
-
-  if my_portfolio.min_weights != None:
-    ef.add_constraint(lambda x : x >= my_portfolio.min_weights)
-  if my_portfolio.max_weights != None:
-    ef.add_constraint(lambda x : x <= my_portfolio.max_weights)
-  weights = ef.max_sharpe()
-  cleaned_weights = ef.clean_weights()
-  wts = cleaned_weights.items()
-
-  result = []
-  for val in wts:
-    a, b = map(list, zip(*[val]))
-    result.append(b)
-  
-
-  if perf==True:
-    pred = ef.portfolio_performance(verbose=True);
-  
-  return flatten(result)
 #-------------------------------------------------------------------------------
 def bl(my_portfolio, perf=True):
 
+
   ohlc = yf.download(my_portfolio.portfolio, start = my_portfolio.start_date, end = my_portfolio.end_date, progress=False)
   market_prices = yf.download("SPY",start = my_portfolio.start_date, end = my_portfolio.end_date, progress=False)["Adj Close"]
   prices = ohlc["Adj Close"]
+
   df = prices.filter(my_portfolio.portfolio)
 
   mcaps = {}
@@ -504,18 +431,19 @@ def bl(my_portfolio, perf=True):
     ef.add_constraint(lambda x : x <= my_portfolio.max_weights)
   ef.max_sharpe()
   weights = ef.clean_weights()
+  lists = dict(weights)
   wts = weights.items()
 
-  result = []
-  for val in wts:
-    a, b = map(list, zip(*[val]))
-    result.append(b)
-  
+
+  wei = []
+  for a in my_portfolio.portfolio:
+    value = lists[a]
+    wei.append(value)
 
   if perf==True:
     pred = ef.portfolio_performance(verbose=True);
   
-  return flatten(result)
+  return wei
 #-------------------------------------------------------------------------------
 def hrp(my_portfolio, perf=True):
 
@@ -676,7 +604,6 @@ def optimizer(my_portfolio, method, vol_max=0.15, pie_size=5, font_size=14):
   plt.legend(l1+l2, loc=2)
   plt.show()
 #-------------------------------------------------------------------------------------------------
-
 def check_schedule(rebalance):
     
     #this is the list of acceptable rebalancing schedules
@@ -701,7 +628,6 @@ def check_schedule(rebalance):
          
     #return that back
     return valid_schedule
-
 #--------------------------------------------------------------------------------
 def valid_range(start_date, end_date, rebalance):
     
@@ -833,9 +759,8 @@ def get_date_range(start_date, end_date, rebalance):
     
     #then we want to return those dates
     return rebalance_dates
-
 #--------------------------------------------------------------------------------
-def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance, maxvol,allocation):
+def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance, allocation, vol_max, div, min,max):
     
     print("")
     print("If failed downloads appear they occur because you are querying for a stock that isn't listed for the desired range, no worries we've worked it out.")
@@ -867,8 +792,11 @@ def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance,
             end_date = dates[i+1],
             portfolio = portfolio_input,
             weights = allocation,
-            max_vol = maxvol,
-            optimizer = "{}".format(optimizer))
+            optimizer = "{}".format(optimizer),
+            max_vol = vol_max,
+            diversification = div,
+            min_weights = min,
+            max_weights = max)
 
         output_df["{}".format(dates[i+1])] = portfolio.weights
         
@@ -878,8 +806,11 @@ def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance,
       start_date = dates[0],
       portfolio = portfolio_input,
       weights = allocation,
-      max_vol = maxvol,
-      optimizer = "{}".format(optimizer))
+      optimizer = "{}".format(optimizer),
+      max_vol = vol_max,
+      diversification = div,
+      min_weights = min,
+      max_weights = max)
     
     output_df['{}'.format(today)] = portfolio.weights
     
